/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 * $Id: not checked in
 *
 * Copyright (C) 2002 Markus Braun
 *
 * All files in this archive are subject to the GNU General Public License.
 * See the file COPYING in the source tree root for full license agreement.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/
#include <lcd.h>
#include "scrollbox.h"
#include <debug.h>
#include <math.h>
#include <stdlib.h>

#ifdef HAVE_LCD_BITMAP
#ifdef SCROLLBOX

// Hacks to get access to other stuff
extern const char fonts[3];
extern const char fontheight[3]; 
extern unsigned char char_gen_6x8[][5];
extern unsigned char char_gen_8x12[][14];
extern unsigned char char_gen_12x16[][22];
extern unsigned char zeros[8];
extern unsigned char ones[8];


// ***** Global variables *****
// Buffer for main scrollbox
unsigned char scrollbox_main_buffer[SCROLLBOXMAIN_BUFFER_WIDTH*SCROLLBOXMAIN_BUFFER_HEIGHT/8];
struct scrollboxinfo scrollbox_main;

// ***** Internal Code *****
void initialize_scrollbox(struct scrollboxinfo *sb, unsigned char *aBuffer, unsigned int aWidth, unsigned int aHeight)
{
    debugf("Initializating scrollbox...\n");
    sb->buffer = aBuffer;
    sb->bufferwidth = aWidth;
    sb->bufferheight = aHeight;
    sb->viewxpos = 16;  // x position on screen of view
    sb->viewypos = 8; // y position on screen of view
    sb->viewwidth = 64; // Width of view on screen
    sb->viewheight = 56;  // height of view on screen
    sb->scrollx = 0; // x start position of buffer to display
    sb->scrolly = 0;// y start position of buffer to display
    sb->visible = 1; // make it show by default!
    
    sb_clear(sb);
}

static void draw_scrollbox(struct scrollboxinfo *sb) 
{
    unsigned int y,x;
    
    if (!sb->visible)
      return; //invisible
    
    for (y=0 ; y<sb->viewheight ; y++)
    {
        for (x=0 ; x<sb->viewwidth; x++)
        {
            if (SB_GET_PIXEL(sb, x+sb->scrollx, y+sb->scrolly))
                DRAW_PIXEL(x+sb->viewxpos,y+sb->viewypos);
            else
                CLEAR_PIXEL(x+sb->viewxpos,y+sb->viewypos);
        } //for x
    }  //for y
} // draw_scrollbox

// ***** "Public" Code *****
void initialize_all_scrollboxes(void)
{
     initialize_scrollbox(&scrollbox_main, 
                          scrollbox_main_buffer, 
                          SCROLLBOXMAIN_BUFFER_WIDTH,
                          SCROLLBOXMAIN_BUFFER_HEIGHT);
}

void draw_all_scrollboxes()
{
    draw_scrollbox(&scrollbox_main);
}

void sb_clear(struct scrollboxinfo *sb)
{
  unsigned int y,x;
  for (y=0 ; y<sb->bufferheight ; y++)
      for (x=0 ; x<sb->bufferwidth; x++)
      {
          SB_CLEAR_PIXEL(sb, x, y);
      }
  sb_drawline(sb, 0,0,50,50);
  sb_putsxy(sb, 10, 0, "George woz ere 2002 and there is nothing to see... move along", 0);
}

// Scrollbox draw line - obviously nicked from lcd_drawline :)
void sb_drawline(struct scrollboxinfo *sb, int x1, int y1, int x2, int y2)
{
    int numpixels;
    int i;
    int deltax, deltay;
    int d, dinc1, dinc2;
    int x, xinc1, xinc2;
    int y, yinc1, yinc2;

    deltax = abs(x2 - x1);
    deltay = abs(y2 - y1);

    if(deltax >= deltay)
    {
        numpixels = deltax;
        d = 2 * deltay - deltax;
        dinc1 = deltay * 2;
        dinc2 = (deltay - deltax) * 2;
        xinc1 = 1;
        xinc2 = 1;
        yinc1 = 0;
        yinc2 = 1;
    }
    else
    {
        numpixels = deltay;
        d = 2 * deltax - deltay;
        dinc1 = deltax * 2;
        dinc2 = (deltax - deltay) * 2;
        xinc1 = 0;
        xinc2 = 1;
        yinc1 = 1;
        yinc2 = 1;
    }
    numpixels++; /* include endpoints */

    if(x1 > x2)
    {
        xinc1 = -xinc1;
        xinc2 = -xinc2;
    }

    if(y1 > y2)
    {
        yinc1 = -yinc1;
        yinc2 = -yinc2;
    }

    x = x1;
    y = y1;

    for(i=0; i<numpixels; i++)
    {
        SB_DRAW_PIXEL(sb, x,y);

        if(d < 0)
        {
            d += dinc1;
            x += xinc1;
            y += yinc1;
        }
        else
        {
            d += dinc2;
            x += xinc2;
            y += yinc2;
        }
    }
} // sb_drawline

void sb_putsxy(struct scrollboxinfo *sb, int x, int y, unsigned char *str, int thisfont)
{
    unsigned int nx = fonts[thisfont];
    unsigned int ny = fontheight[thisfont];
    int ch;
    unsigned char *src;
    int lcd_x = x;
    int lcd_y = y;

    while (((ch = *str++) != '\0') && (lcd_x + nx < sb->bufferwidth))
    {
        if (lcd_y + ny > LCD_HEIGHT)
            return;
//        debugf("Draw %c (%d) which is %dx%d\n", ch, ch,nx,ny);
        /* Limit to char generation table */
        if ((ch < ASCII_MIN) || (ch > ASCII_MAX))
            /* replace unsupported letters with question marks */
            ch = '?' - ASCII_MIN;
        else
            ch -= ASCII_MIN;
        
        if (thisfont == 2)
            src = char_gen_12x16[ch];
        else if (thisfont == 1)
            src = char_gen_8x12[ch];
        else
            src = char_gen_6x8[ch];
        
        sb_bitmap (sb, src, lcd_x, lcd_y, nx-1, ny, true);
        sb_bitmap (sb, zeros, lcd_x+nx-1, lcd_y, 1, ny, true);

        lcd_x += nx;
    }
} // sb_putsxy

void sb_bitmap (struct scrollboxinfo *sb, unsigned char *src, int x, int y, int nx, int ny, bool clear)
{
    unsigned char *dst;
    unsigned char *dst2;
    unsigned int data, mask, mask2, mask3, mask4;
    int shift;

//    debugf("Drawing bitmap %d at %d,%d size %d,%d\n", src, x, y, nx, ny);
    if (((unsigned)x >= sb->bufferwidth) || ((unsigned)y >= sb->bufferwidth))
        return;
    if (((unsigned)(x + nx)) >= sb->bufferwidth)
       nx = sb->bufferwidth - x;
    if (((unsigned)(y + ny)) >= sb->bufferheight)
       ny = sb->bufferheight - y;      

    shift = y & 7;
    
    dst2 = &sb->buffer[SB_BUFFER_OFFSET(sb,x,y)];
    ny += shift;

    /* Calculate bit masks */
    mask4 = ~(0xfe << ((ny-1) & 7));
    if (clear)
    {
        mask = ~(0xff << shift);
        mask2 = 0;
        mask3 = ~mask4;
        if (ny <= 8)
            mask3 |= mask;
    }
    else
        mask = mask2 = mask3 = 0xff;

    /* Loop for each column */
    for (x = 0; x < nx; x++)
    {
        dst = dst2;
        //debugf("Drawing row at %d\n",dst);
        dst2 += sb->bufferheight/8;
        data = 0;
        y = 0;

        if (ny > 8)
        {
            /* First partial row */
            data = *src++ << shift;
            *dst = (*dst & mask) | data;
            data >>= 8;
            dst++;

            /* Intermediate rows */
            for (y = 8; y < ny-8; y += 8)
            {
                data |= *src++ << shift;
                *dst = (*dst & mask2) | data;
                data >>= 8;
                dst++;
            }
        }

        /* Last partial row */
        if (y + shift < ny)
            data |= *src++ << shift;
        *dst = (*dst & mask3) | (data & mask4);
    }
} // sb_bitmap

#endif // SCROLLBOX
#endif // HAVE_LCD_BITMAP
