/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 * $Id: not checked in
 *
 * Copyright (C) 2002 Markus Braun
 *
 * All files in this archive are subject to the GNU General Public License.
 * See the file COPYING in the source tree root for full license agreement.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/
#ifndef __SCROLLBOX_H__
#define __SCROLLBOX_H__
#include <lcd.h>

#ifdef HAVE_LCD_BITMAP
#ifdef SCROLLBOX
/* Scrollbox stuff started by George Styles */

// ***** TypeDefs *****
struct scrollboxinfo {
    char *buffer;  // Pointer to buffer
    unsigned char visible;     
    unsigned int bufferwidth; // Width of buffer
    unsigned int bufferheight;  // Height of buffer
    unsigned int viewxpos;  // x position on screen of view
    unsigned int viewypos; // y position on screen of view
    unsigned int viewwidth; // Width of view on screen
    unsigned int viewheight;  // height of view on screen
    unsigned int scrollx; // x start position of buffer to display
    unsigned int scrolly; // y start position of buffer to display
//    unsigned int scrollcount; // countdown to next scroll
//    unsigned int scrollspeed; // number to reset scrollcount to after a scroll, ie frequency
};

// ***** Defines *****
#define SCROLLBOXMAIN_BUFFER_WIDTH 112*4 // 4 screens wide
#define SCROLLBOXMAIN_BUFFER_HEIGHT 128 // 2 screen high (FIXME)

// ****** Public Functions *****
extern void initialize_all_scrollboxes(void);
extern void draw_all_scrollboxes();
extern void sb_clear(struct scrollboxinfo *sb);
extern void sb_drawline(struct scrollboxinfo *sb, int x1, int y1, int x2, int y2);
extern void sb_putsxy(struct scrollboxinfo *sb, int x, int y, unsigned char *str, int thisfont);
extern void sb_bitmap (struct scrollboxinfo *sb, unsigned char *src, int x, int y, int nx, int ny, bool clear);

// ***** "Public" Variables *****
extern struct scrollboxinfo scrollbox_main;

// ***** Macros *****
#define SB_BUFFER_OFFSET(sb, x, y) ((y)/8+( ( (x)*sb->bufferheight)/8 ))
#define SB_BUFFER_BYTE(sb, x, y) (sb->buffer[(SB_BUFFER_OFFSET(sb,x,y))])
#define SB_BUFFER_MASK(sb, x, y) (1<< ( (y)&7))
#define SB_DRAW_PIXEL(sb, x, y) SB_BUFFER_BYTE(sb,x,y) |= SB_BUFFER_MASK(sb,x,y)
#define SB_CLEAR_PIXEL(sb, x, y) SB_BUFFER_BYTE(sb,x,y) &= ~SB_BUFFER_MASK(sb,x,y)
#define SB_GET_PIXEL(sb, x, y) (SB_BUFFER_BYTE(sb,x,y) & SB_BUFFER_MASK(sb,x,y))

#endif /* SCROLLBOX */
#endif /* HAVE_LCD_BITMAP */
#endif /* __SCROLLBOX_H__ */
